unit uOptions;
{$INCLUDE 'Plugin.inc'}

interface

uses
  SysUtils,
  {$INCLUDE 'DelphiFarUnits.inc'} ;

type
  TOptions = class(TFarConfig)
  private
    fProcessSubfolders: boolean;
    fUseMaxScanDepth: boolean;
    fMaxScanDepth: integer;
    fProcessSelected: boolean;
    fCompareTime: boolean;
    fLowPrecisionTime: boolean;
    fIgnorePossibleTimeZoneDifferences: boolean;
    fCompareSize: boolean;
    fCompareFileNameCase: boolean;
    fCompareContents: boolean;
    fCompareContentsIgnore: boolean;
    fIgnoreNewLines: boolean;
    fIgnoreWhitespace: boolean;
    fSelectDifferencesOnBothPanels: boolean;
    fCompareCommand: string;
    fCompareCommandArguments: string;
    fCompareCommandAcceptsNTFileNames: boolean;
    fSilentModeByDefault: boolean;
    fShowDescriptionIsSyncFile: boolean;
    fGlobalRemember: boolean;
    fDisableInternalHotkeys: boolean;
    fChangeDirectionInIncompleteBlocks: boolean;
    procedure SetCompareCommand(const Value: string);
    procedure SetCompareCommandAcceptsNTFileNames(const Value: boolean);
    procedure SetCompareCommandArguments(const Value: string);
    procedure SetCompareContents(const Value: boolean);
    procedure SetCompareContentsIgnore(const Value: boolean);
    procedure SetCompareFileNameCase(const Value: boolean);
    procedure SetCompareSize(const Value: boolean);
    procedure SetCompareTime(const Value: boolean);
    procedure SetDisableInternalHotkeys(const Value: boolean);
    procedure SetGlobalRemember(const Value: boolean);
    procedure SetChangeDirectionInIncompleteBlocks(const Value: boolean);
    procedure SetIgnoreNewLines(const Value: boolean);
    procedure SetIgnorePossibleTimeZoneDifferences(const Value: boolean);
    procedure SetIgnoreWhitespace(const Value: boolean);
    procedure SetLowPrecisionTime(const Value: boolean);
    procedure SetMaxScanDepth(const Value: integer);
    procedure SetProcessSelected(const Value: boolean);
    procedure SetProcessSubfolders(const Value: boolean);
    procedure SetSelectDifferencesOnBothPanels(const Value: boolean);
    procedure SetShowDescriptionIsSyncFile(const Value: boolean);
    procedure SetSilentModeByDefault(const Value: boolean);
    procedure SetUseMaxScanDepth(const Value: boolean);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure Load;
    procedure Save(Force: boolean = False);
    procedure Assign(Options: TOptions);
    property ProcessSubfolders: boolean read fProcessSubfolders write SetProcessSubfolders;
    property UseMaxScanDepth: boolean read fUseMaxScanDepth write SetUseMaxScanDepth;
    property MaxScanDepth: integer read fMaxScanDepth write SetMaxScanDepth;
    property ProcessSelected: boolean read fProcessSelected write SetProcessSelected;
    property CompareTime: boolean read fCompareTime write SetCompareTime;
    property LowPrecisionTime: boolean read fLowPrecisionTime write SetLowPrecisionTime;
    property IgnorePossibleTimeZoneDifferences: boolean read fIgnorePossibleTimeZoneDifferences write SetIgnorePossibleTimeZoneDifferences;
    property CompareSize: boolean read fCompareSize write SetCompareSize;
    property CompareFileNameCase: boolean read fCompareFileNameCase write SetCompareFileNameCase;
    property CompareContents: boolean read fCompareContents write SetCompareContents;
    property CompareContentsIgnore: boolean read fCompareContentsIgnore write SetCompareContentsIgnore;
    property IgnoreNewLines: boolean read fIgnoreNewLines write SetIgnoreNewLines;
    property IgnoreWhitespace: boolean read fIgnoreWhitespace write SetIgnoreWhitespace;
    property SelectDifferencesOnBothPanels: boolean read fSelectDifferencesOnBothPanels write SetSelectDifferencesOnBothPanels;
    property CompareCommand: string read fCompareCommand write SetCompareCommand;
    property CompareCommandArguments: string read fCompareCommandArguments write SetCompareCommandArguments;
    property CompareCommandAcceptsNTFileNames: boolean read fCompareCommandAcceptsNTFileNames write SetCompareCommandAcceptsNTFileNames;
    property SilentModeByDefault: boolean read fSilentModeByDefault write SetSilentModeByDefault;
    property ShowDescriptionIsSyncFile: boolean read fShowDescriptionIsSyncFile write SetShowDescriptionIsSyncFile;
    property GlobalRemember: boolean read fGlobalRemember write SetGlobalRemember;
    property DisableInternalHotkeys: boolean read fDisableInternalHotkeys write SetDisableInternalHotkeys;
    property ChangeDirectionInIncompleteBlocks: boolean read fChangeDirectionInIncompleteBlocks write SetChangeDirectionInIncompleteBlocks;
  end;

implementation

{ TOptions }

procedure TOptions.Assign(Options: TOptions);
begin
  ProcessSubfolders := Options.ProcessSubfolders;
  UseMaxScanDepth := Options.UseMaxScanDepth;
  MaxScanDepth := Options.MaxScanDepth;
  ProcessSelected := Options.ProcessSelected;
  CompareTime := Options.CompareTime;
  LowPrecisionTime := Options.LowPrecisionTime;
  IgnorePossibleTimeZoneDifferences := Options.IgnorePossibleTimeZoneDifferences;
  CompareSize := Options.CompareSize;
  CompareFileNameCase := Options.CompareFileNameCase;
  CompareContents := Options.CompareContents;
  CompareContentsIgnore := Options.CompareContentsIgnore;
  IgnoreNewLines := Options.IgnoreNewLines;
  IgnoreWhitespace := Options.IgnoreWhitespace;
  SelectDifferencesOnBothPanels := Options.SelectDifferencesOnBothPanels;
  CompareCommand := Options.CompareCommand;
  CompareCommandArguments := Options.CompareCommandArguments;
  CompareCommandAcceptsNTFileNames := Options.CompareCommandAcceptsNTFileNames;
  SilentModeByDefault := Options.SilentModeByDefault;
  ShowDescriptionIsSyncFile := Options.ShowDescriptionIsSyncFile;
  GlobalRemember := Options.GlobalRemember;
  DisableInternalHotkeys := Options.DisableInternalHotkeys;
  ChangeDirectionInIncompleteBlocks := Options.ChangeDirectionInIncompleteBlocks;
end;

constructor TOptions.Create;
begin
  inherited Create('DirSync', False);
  Init;
end;

destructor TOptions.Destroy;
begin
  inherited;
end;

procedure TOptions.Init;
begin
  ProcessSubfolders := False;
  UseMaxScanDepth := False;
  MaxScanDepth := 99;
  ProcessSelected := False;
  CompareTime := True;
  LowPrecisionTime := True;
  IgnorePossibleTimeZoneDifferences := True;
  CompareSize := True;
  CompareFileNameCase := False;
  CompareContents := False;
  CompareContentsIgnore := False;
  IgnoreNewLines := True;
  IgnoreWhitespace := False;
  SelectDifferencesOnBothPanels := False;
  CompareCommand := 'winmerge';
  CompareCommandArguments := '${LEFTFILE} ${RIGHTFILE}';
  CompareCommandAcceptsNTFileNames := False; // WinMerge does not support them
  SilentModeByDefault := False;
  ShowDescriptionIsSyncFile := True;
  GlobalRemember := False;
  DisableInternalHotkeys := False;
  ChangeDirectionInIncompleteBlocks := False;
  SetChanged;
end;

procedure TOptions.Load;
begin
  if Open then
    try
      ProcessSubfolders := ReadBoolean('ProcessSubfolders', ProcessSubfolders);
      UseMaxScanDepth := ReadBoolean('UseMaxScanDepth', UseMaxScanDepth);
      MaxScanDepth := ReadInteger('MaxScanDepth', MaxScanDepth);
      ProcessSelected := ReadBoolean('ProcessSelected', ProcessSelected);
      CompareTime := ReadBoolean('CompareTime', CompareTime);
      LowPrecisionTime := ReadBoolean('LowPrecisionTime', LowPrecisionTime);
      IgnorePossibleTimeZoneDifferences := ReadBoolean('IgnorePossibleTimeZoneDifferences', IgnorePossibleTimeZoneDifferences);
      CompareSize := ReadBoolean('CompareSize', CompareSize);
      CompareFileNameCase := ReadBoolean('CompareFileNameCase', CompareFileNameCase);
      CompareContents := ReadBoolean('CompareContents', CompareContents);
      CompareContentsIgnore := ReadBoolean('CompareContentsIgnore', CompareContentsIgnore);
      IgnoreNewLines := ReadBoolean('IgnoreNewLines', IgnoreNewLines);
      IgnoreWhitespace := ReadBoolean('IgnoreWhitespace', IgnoreWhitespace);
      SelectDifferencesOnBothPanels := ReadBoolean('SelectDifferencesOnBothPanels', SelectDifferencesOnBothPanels);
      CompareCommand := ReadString('CompareCommand', CompareCommand);
      CompareCommandArguments := ReadString('CompareCommandArguments', CompareCommandArguments);
      CompareCommandAcceptsNTFileNames := ReadBoolean('CompareCommandAcceptsNTFileNames', CompareCommandAcceptsNTFileNames);
      SilentModeByDefault := ReadBoolean('SilentModeByDefault', SilentModeByDefault);
      ShowDescriptionIsSyncFile := ReadBoolean('ShowDescriptionIsSyncFile', ShowDescriptionIsSyncFile);
      GlobalRemember := ReadBoolean('GlobalRemember', GlobalRemember);
      DisableInternalHotkeys := ReadBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      ChangeDirectionInIncompleteBlocks := ReadBoolean('ChangeDirectionInIncompleteBlocks', ChangeDirectionInIncompleteBlocks);
      ClearChanged;
    finally
      Close;
    end;
end;

procedure TOptions.Save(Force: boolean);
begin
  if not (Force or Changed) then
    Exit;
  if Open then
    try
      WriteBoolean('ProcessSubfolders', ProcessSubfolders);
      WriteBoolean('UseMaxScanDepth', UseMaxScanDepth);
      WriteInteger('MaxScanDepth', MaxScanDepth);
      WriteBoolean('ProcessSelected', ProcessSelected);
      WriteBoolean('CompareTime', CompareTime);
      WriteBoolean('LowPrecisionTime', LowPrecisionTime);
      WriteBoolean('IgnorePossibleTimeZoneDifferences', IgnorePossibleTimeZoneDifferences);
      WriteBoolean('CompareSize', CompareSize);
      WriteBoolean('CompareFileNameCase', CompareFileNameCase);
      WriteBoolean('CompareContents', CompareContents);
      WriteBoolean('CompareContentsIgnore', CompareContentsIgnore);
      WriteBoolean('IgnoreNewLines', IgnoreNewLines);
      WriteBoolean('IgnoreWhitespace', IgnoreWhitespace);
      WriteBoolean('SelectDifferencesOnBothPanels', SelectDifferencesOnBothPanels);
      WriteString('CompareCommand', CompareCommand);
      WriteString('CompareCommandArguments', CompareCommandArguments);
      WriteBoolean('CompareCommandAcceptsNTFileNames', CompareCommandAcceptsNTFileNames);
      WriteBoolean('SilentModeByDefault', SilentModeByDefault);
      WriteBoolean('ShowDescriptionIsSyncFile', ShowDescriptionIsSyncFile);
      WriteBoolean('GlobalRemember', GlobalRemember);
      WriteBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      WriteBoolean('ChangeDirectionInIncompleteBlocks', ChangeDirectionInIncompleteBlocks);
      ClearChanged;
    finally
      Close;
    end;
end;

procedure TOptions.SetChangeDirectionInIncompleteBlocks(const Value: boolean);
begin
  SetConfigOption(fChangeDirectionInIncompleteBlocks, Value);
end;

procedure TOptions.SetCompareCommand(const Value: string);
begin
  SetConfigOption(fCompareCommand, Value);
end;

procedure TOptions.SetCompareCommandAcceptsNTFileNames(const Value: boolean);
begin
  SetConfigOption(fCompareCommandAcceptsNTFileNames, Value);
end;

procedure TOptions.SetCompareCommandArguments(const Value: string);
begin
  SetConfigOption(fCompareCommandArguments, Value);
end;

procedure TOptions.SetCompareContents(const Value: boolean);
begin
  SetConfigOption(fCompareContents, Value);
end;

procedure TOptions.SetCompareContentsIgnore(const Value: boolean);
begin
  SetConfigOption(fCompareContentsIgnore, Value);
end;

procedure TOptions.SetCompareFileNameCase(const Value: boolean);
begin
  SetConfigOption(fCompareFileNameCase, Value);
end;

procedure TOptions.SetCompareSize(const Value: boolean);
begin
  SetConfigOption(fCompareSize, Value);
end;

procedure TOptions.SetCompareTime(const Value: boolean);
begin
  SetConfigOption(fCompareTime, Value);
end;

procedure TOptions.SetDisableInternalHotkeys(const Value: boolean);
begin
  SetConfigOption(fDisableInternalHotkeys, Value);
end;

procedure TOptions.SetGlobalRemember(const Value: boolean);
begin
  SetConfigOption(fGlobalRemember, Value);
end;

procedure TOptions.SetIgnoreNewLines(const Value: boolean);
begin
  SetConfigOption(fIgnoreNewLines, Value);
end;

procedure TOptions.SetIgnorePossibleTimeZoneDifferences(const Value: boolean);
begin
  SetConfigOption(fIgnorePossibleTimeZoneDifferences, Value);
end;

procedure TOptions.SetIgnoreWhitespace(const Value: boolean);
begin
  SetConfigOption(fIgnoreWhitespace, Value);
end;

procedure TOptions.SetLowPrecisionTime(const Value: boolean);
begin
  SetConfigOption(fLowPrecisionTime, Value);
end;

procedure TOptions.SetMaxScanDepth(const Value: integer);
begin
  SetConfigOption(fMaxScanDepth, Value);
end;

procedure TOptions.SetProcessSelected(const Value: boolean);
begin
  SetConfigOption(fProcessSelected, Value);
end;

procedure TOptions.SetProcessSubfolders(const Value: boolean);
begin
  SetConfigOption(fProcessSubfolders, Value);
end;

procedure TOptions.SetSelectDifferencesOnBothPanels(const Value: boolean);
begin
  SetConfigOption(fSelectDifferencesOnBothPanels, Value);
end;

procedure TOptions.SetShowDescriptionIsSyncFile(const Value: boolean);
begin
  SetConfigOption(fShowDescriptionIsSyncFile, Value);
end;

procedure TOptions.SetSilentModeByDefault(const Value: boolean);
begin
  SetConfigOption(fSilentModeByDefault, Value);
end;

procedure TOptions.SetUseMaxScanDepth(const Value: boolean);
begin
  SetConfigOption(fUseMaxScanDepth, Value);
end;

end.
